from numpy import pi, cos, zeros

class Davies:
    def __init__(self,N1,N2,x_i,y_i,x_e,y_e):
        self.N1, self.N2 = N1, N2
        self.beta_i = self.mask(x_i,y_i)
        self.beta_e = self.mask(x_e,y_e)
        self.x_e,self.y_e = x_e,y_e
    def mask0(self,c,c0,delta): # 1D building block for Davies relaxation
        N1, N2 = self.N1, self.N2
        m = zeros(c.size)
        c1 = c0+N2*delta
        for i in range(c.size):
            ci=c[i]
            m[i] = (1.+cos((ci-c0)*pi/(N2*delta)))/2.0
            if ci < c0 : m[i]=1.
            if ci > c1 : m[i]=0.
        return m
    def relax(self, llm, step, flow):
        beta_i, beta_e = self.beta_i, self.beta_e
        m,S,u,Phi,W=flow
        for l in range(llm):
            step.mass[:,l]        = beta_i*step.mass[:,l]        + (1.-beta_i)*m[:,l]
            step.theta_rhodz[:,l] = beta_i*step.theta_rhodz[:,l] + (1.-beta_i)*S[:,l]
            step.u[:,l]           = beta_e*step.u[:,l]           + (1.-beta_e)*u[:,l]
        for l in range(llm+1):
            step.geopot[:,l]      = beta_i*step.geopot[:,l]      + (1.-beta_i)*Phi[:,l]
            step.W[:,l]           = beta_i*step.W[:,l]           + (1.-beta_i)*W[:,l]
